package com.ys.bdtp.adm.mvvm.vm.project

import android.util.Log
import com.ys.bdtp.adm.mvvm.model.db.AdmDatabase
import com.ys.bdtp.adm.mvvm.model.db.entity.UserEntity
import com.ys.bdtp.adm.mvvm.model.db.entity.relation.UserWithProjects
import com.ys.bdtp.adm.mvvm.model.nt.Api
import com.ys.bdtp.adm.mvvm.model.nt.Fail
import com.ys.bdtp.adm.mvvm.model.nt.request
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.withContext

/**
 * ProjectRepository
 *
 * @author bai
 *
 * @param
 */
class ProjectRepository(private val api: Api, private val db: AdmDatabase) {
    /**
     * 获取账号信息
     */
    suspend fun requestAccount(password: String, failure: (Fail) -> Unit) {
        request(
            request = { api.getAccount() },
            success = { account ->
                val user = UserEntity(
                    password = password,
                    state = 1,
                    id = account.id,
                    account = account.account,
                    deptId = account.deptId,
                    email = account.email,
                    phone = account.phone,
                    name = account.name,
                )
                val users = db.userDao().getUsers().onEach { u -> u.state = 0 }.toMutableList()
                users.add(user)
                db.userDao().insUsers(users)
                account.projects.forEach { it.userId = user.id }
                db.projectDao().insProjects(account.projects)
                Log.d("requestAccount", "requestAccount success")
            },
            failure = { fail ->
                failure(fail)
                Log.d("requestAccount", "requestAccount failure : $fail")
            }
        )
    }

    /**
     *  获取用户以及用户项目
     */
    fun getUserWithProjects(): Flow<UserWithProjects> {
        return db.userDao().getUserWithProjects()
    }

    suspend fun logout() {
        withContext(Dispatchers.IO) {
            val user = db.userDao().currentUser()
            if (user != null) {
                user.state = 0
                db.userDao().updUser(user)
            }
        }
    }
}